@@ -1,7 +1,7 @@
 /* SPIM S20 MIPS simulator.
    Execute SPIM instructions.
 
-   Copyright (C) 1990-1997 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-1998 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
    Changes for DOS and Windows versions by David A. Carley (dac@cs.wisc.edu)
 
@@ -21,7 +21,7 @@
    PURPOSE. */
 
 
-/* $Header: /u/l/a/larus/Software/SPIM/src/RCS/run.c,v 3.47 1998/01/14 20:57:44 larus Exp $
+/* $Header: /u/l/a/larus/Software/SPIM/src/RCS/run.c,v 3.48 1999/01/08 00:32:54 larus Exp $
 */
 
 
@@ -253,24 +253,24 @@ run_spim (initial_PC, steps_to_run, display)
 	    case Y_BC2F_OP:
 	    case Y_BC3F_OP:
 	      BRANCH_INST (CpCond[OPCODE (inst) - Y_BC0F_OP] == 0,
-			   PC + (SIGN_EX (IOFFSET (inst)) << 2));
+			   PC + IDISP (inst));
 	      break;
 
 	    case Y_BC0T_OP:
 	    case Y_BC2T_OP:
 	    case Y_BC3T_OP:
 	      BRANCH_INST (CpCond[OPCODE (inst) - Y_BC0T_OP] != 0,
-			   PC + (SIGN_EX (IOFFSET (inst)) << 2));
+			   PC + IDISP (inst));
 	      break;
 
 	    case Y_BEQ_OP:
 	      BRANCH_INST (R[RS (inst)] == R[RT (inst)],
-			   PC + (SIGN_EX (IOFFSET (inst)) << 2));
+			   PC + IDISP (inst));
 	      break;
 
 	    case Y_BGEZ_OP:
 	      BRANCH_INST (SIGN_BIT (R[RS (inst)]) == 0,
-			   PC + (SIGN_EX (IOFFSET (inst)) << 2));
+			   PC + IDISP (inst));
 	      break;
 
 	    case Y_BGEZAL_OP:
@@ -279,22 +279,22 @@ run_spim (initial_PC, steps_to_run, display)
 	      else
 		R[31] = PC + BYTES_PER_WORD;
 	      BRANCH_INST (SIGN_BIT (R[RS (inst)]) == 0,
-			   PC + (SIGN_EX (IOFFSET (inst)) << 2));
+			   PC + IDISP (inst));
 	      break;
 
 	    case Y_BGTZ_OP:
 	      BRANCH_INST (R[RS (inst)] != 0 && SIGN_BIT (R[RS (inst)]) == 0,
-			   PC + (SIGN_EX (IOFFSET (inst)) << 2));
+			   PC + IDISP (inst));
 	      break;
 
 	    case Y_BLEZ_OP:
 	      BRANCH_INST (R[RS (inst)] == 0 || SIGN_BIT (R[RS (inst)]) != 0,
-			   PC + (SIGN_EX (IOFFSET (inst)) << 2));
+			   PC + IDISP (inst));
 	      break;
 
 	    case Y_BLTZ_OP:
 	      BRANCH_INST (SIGN_BIT (R[RS (inst)]) != 0,
-			   PC + (SIGN_EX (IOFFSET (inst)) << 2));
+			   PC + IDISP (inst));
 	      break;
 
 	    case Y_BLTZAL_OP:
@@ -303,12 +303,12 @@ run_spim (initial_PC, steps_to_run, display)
 	      else
 		R[31] = PC + BYTES_PER_WORD;
 	      BRANCH_INST (SIGN_BIT (R[RS (inst)]) != 0,
-			   PC + (SIGN_EX (IOFFSET (inst)) << 2));
+			   PC + IDISP (inst));
 	      break;
 
 	    case Y_BNE_OP:
 	      BRANCH_INST (R[RS (inst)] != R[RT (inst)],
-			   PC + (SIGN_EX (IOFFSET (inst)) << 2));
+			   PC + IDISP (inst));
 	      break;
 
 	    case Y_BREAK_OP:
@@ -890,12 +890,12 @@ run_spim (initial_PC, steps_to_run, display)
 
 	    case Y_BC1F_OP:
 	      BRANCH_INST (FpCond == 0,
-			   PC + (SIGN_EX (IOFFSET (inst)) << 2));
+			   PC + IDISP (inst));
 	      break;
 
 	    case Y_BC1T_OP:
 	      BRANCH_INST (FpCond == 1,
-			   PC + (SIGN_EX (IOFFSET (inst)) << 2));
+			   PC + IDISP (inst));
 	      break;
 
 	    case Y_C_F_S_OP:
