@@ -1,7 +1,7 @@
 /* SPIM S20 MIPS simulator.
    Terminal interface for SPIM simulator.
 
-   Copyright (C) 1990-1997 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-1998 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
    Changes for DOS and Windows versions by David A. Carley (dac@cs.wisc.edu)
 
@@ -21,7 +21,7 @@
    PURPOSE. */
 
 
-/* $Header: /u/l/a/larus/Software/SPIM/src/RCS/spim.c,v 3.63 1998/01/14 20:45:06 larus Exp $
+/* $Header: /u/l/a/larus/Software/SPIM/src/RCS/spim.c,v 3.65 1999/01/08 00:32:54 larus Exp $
 */
 
 
@@ -49,6 +49,8 @@
 #define termio termios
 #endif
 
+
+#ifndef WIN32
 #include <sys/time.h>
 
 #ifdef USE_TERMIO
@@ -60,6 +62,7 @@
 #include <sys/ioctl.h>
 #include <sgtty.h>
 #endif
+#endif
 
 #ifdef __STDC__
 #include <stdarg.h>
@@ -236,8 +239,18 @@ main (argc, argv)
       console_to_program ();
       initialize_run_stack (argc - argv_ptr, &argv[argv_ptr]);
       if (!setjmp (spim_top_level_env))
+      {
+	char *undefs = undefined_symbol_string ();
+	if (undefs != NULL)
+	  {
+	    write_output (message_out, "The following symbols are undefined:\n");
+	    write_output (message_out, undefs);
+	    write_output (message_out, "\n");
+	    free (undefs);
+	  }
 	run_program (find_symbol_address (DEFAULT_RUN_LOCATION),
 		     DEFAULT_RUN_STEPS, 0, 0);
+      }
       console_to_spim ();
     }
 
@@ -360,9 +373,20 @@ parse_spim_command (file, redo)
 	initialize_run_stack (0, 0);
 	console_to_program ();
 	if (addr)
+	{
+	  char *undefs = undefined_symbol_string ();
+	  if (undefs != NULL)
+	    {
+	      write_output (message_out, "The following symbols are undefined:\n");
+	      write_output (message_out, undefs);
+	      write_output (message_out, "\n");
+	      free (undefs);
+	    }
+
 	  if (run_program (addr, DEFAULT_RUN_STEPS, 0, 0))
 	    write_output (message_out, "Breakpoint encountered at 0x%08x\n",
 			  PC);
+	}
 	console_to_spim ();
 
 	prev_cmd = RUN_CMD;
